/*
 Mini Temp Logger
 Version: Prototype R4
 Author: Alex from insideGadgets (www.insidegadgets.com)
 Created: 22/02/2015
 Last Modified: 11/09/2016
 
 Used to log temperature using the TMP102 sensor every x seconds/minutes/hours/days which is saved to an external 
 1Mbit I2C EEPROM. You can transfer the data via USB to your computer using the transfer program.
 
 Remember to change your fuse bits so the ATtiny84 uses 12MHz crystal and divide clock by 8
 avrdude -p ATtiny84 -c usbtiny -U lfuse:w:0x4f:m -U hfuse:w:0xdf:m -U efuse:w:0xff:m
 
 */

// ATtiny24/44/84 Pin map
//
//                            +-\/-+
//                      VCC  1|o   |14 GND
//                 XTAL PB0  2|    |13 PA0 USB D+ 
//                 XTAL PB1  3|    |12 PA1 USB D-
//                Reset PB3  4|    |11 PA2 3.3V LDO Sense
//               USB D+ PB2  5|    |10 PA3 RTC 1Hz Sense
//                  LED PA7  6|    |9  PA4 SDA
// Button/Voltage Sense PA6  7|    |8  PA5 SCL
//                            +----+

#define F_CPU 12000000 // If this is changed, modify it in usbconfig.h too

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <util/delay.h>
#include <string.h>
#include "usbdrv/usbdrv.h"
#include "usbdrv/oddebug.h"
#include "usbconfig.h"
#include "setup.c" // See defines, variables, constants, functions here

int main (void) {
	setup();
	
	// debug
	//logging = true;
	
	while (1) {
		// USB plugged in - transfer data
		if (PINA & (1<<usbSensePin)) {
			// Disable pin change interrupt
			cbi(GIMSK, PCIE0);
			
			// Reset data count and set I2C delay cycles to USB mode
			dataCount = 0;
			I2C_DELAY_CYCLES = I2C_DELAY_CYCLES_USB;
			
			// Increase clock speed to 12MHz
			CLKPR = (1<<CLKPCE); // Prescaler enable
			CLKPR = 0; // Clock division factor 1 (0000)
			
			startUSB();
			inUSB = true;
			while (1) {
				usbPoll();
				PORTA |= (1<<ledPin);
				
				// Check if USB was disconnected
				if (!(PINA & (1<<usbSensePin))) {
					break;
				}
			}
			
			// Decrease clock speed to 1.5MHz
			CLKPR = (1<<CLKPCE); // Prescaler enable
			CLKPR = (1<<CLKPS1) | (1<<CLKPS0); // Clock division factor 8 (0011)
			
			PORTA &= ~(1<<ledPin);
			inUSB = false;
			
			// Enable pin change interrupt
			sbi(GIMSK, PCIE0);
		}
		
		// Start logging
		else if (logging == true) { 
			PORTA &= ~(1<<ledPin);
			
			// Disable pin change on 3.3V LDO sense and enable RTC 1Hz sense
			cbi(PCMSK0, PCINT2);
			sbi(PCMSK0, PCINT3);
			
			// Reset data count and EEPROM variables and set I2C delay cycles to battery mode
			dataCount = 0;
			eepromBufferCounter = 0;
			eepromPageCounter = 0;
			eepromPageByteCounter = 0;
			I2C_DELAY_CYCLES = I2C_DELAY_CYCLES_BAT;
			
			// Copy RTC time to ATtiny EEPROM so we know the start date/time
			save_rtc_time_to_eeprom();
			
			// Set clock output at 1Hz
			soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_CONTROL, NOREPEATSTART);
			soft_i2c_master_write(RTC_CONTROL_SQWEN_BIT);
			soft_i2c_master_stop();
			
			// Start the RTC (if it's not already started, this means the RTC stopped or RTC time hasn't been programmed)
			soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_RTCSEC, REPEATSTART);
			uint8_t isRtcRunning = soft_i2c_master_read(1);
			soft_i2c_master_stop();
			if (!(isRtcRunning & RTC_RTCSEC_ST_BIT)) {
				soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_RTCSEC, NOREPEATSTART);
				soft_i2c_master_write(RTC_RTCSEC_ST_BIT);
				soft_i2c_master_stop();
			}
			
			// Write first blank page on EEPROM
			soft_i2c_eeprom_page_blank(EEPROM_ADDR, eepromPageCounter);
			
			// Read delay time
			uint8_t delaySeconds = eeprom_read_byte((uint8_t*) delayTimeLocation);
			uint8_t delayMinutes = eeprom_read_byte((uint8_t*) delayTimeLocation+1);
			uint8_t delayHours = eeprom_read_byte((uint8_t*) delayTimeLocation+2);
			uint8_t delayDays = eeprom_read_byte((uint8_t*) delayTimeLocation+3);
			
			// Unprogrammed delay time?
			if (delaySeconds == blankEeprombyte) {
				delaySeconds = 0;
				delayMinutes = 5; // Set 5 minutes as the default delay time
				delayHours = 0;
				delayDays = 0;
			}
			
			//int tempResult = 1000; // debugging
			
			while (logging == true) {
				PORTA |= (1<<ledPin); // Turn on LED
				int tempResult = tmp102_read_temp(TEMP_ADDR); // Read TMP102
				//watchdog_sleep(T32MS); // Show LED lit for a little bit (debugging)
				PORTA &= ~(1<<ledPin); // Turn off
				// watchdog_sleep(T32MS); // debugging
				//tempResult++; // debugging
				
				eepromBuffer[eepromBufferCounter++] = (tempResult >> 8);
				eepromBuffer[eepromBufferCounter++] = (tempResult & 0xFF);
				
				// Once our 32 byte buffer is full, write it to the I2C EEPROM
				if (eepromBufferCounter >= EEPROM_BUFFER_LENGTH) {
					soft_i2c_eeprom_page_write(EEPROM_ADDR, eepromPageCounter, eepromPageByteCounter, eepromBuffer, EEPROM_BUFFER_LENGTH);
					eepromPageByteCounter += eepromBufferCounter;
					eepromBufferCounter = 0;
					
					// If we have almost filled up the current page (224 out of 256 bytes) and we aren't on the last page of the EEPROM (511)
					// then blank the next page 
					if (eepromPageByteCounter == 224 && eepromPageCounter < 511) {
						watchdog_sleep(T16MS); // Wait for page write before to finish
						soft_i2c_eeprom_page_blank(EEPROM_ADDR, eepromPageCounter+1);
					}
					
					// Increment page address
					if (eepromPageByteCounter == 0) { 
						if (eepromPageCounter >= 511) { // EEPROM full, sleep until USB plugged in or button pressed again
							logging = false;
						}
						eepromPageCounter++;
						eepromPageByteCounter = 0;
					}
				}
				
				// Wait delay time
				uint16_t delayCount = delaySeconds + (delayMinutes * 60) + (delayHours * 3600);
				while (delayCount != 0 && logging == true) {
					rtc_sleep_1_sec();
					delayCount--;
				}
				if (delayDays >= 1) {
					uint32_t delayCount = delayDays * 86400;
					while (delayCount != 0 && logging == true) {
						rtc_sleep_1_sec();
						delayCount--;
					}
				}
			}
			
			// Button was held to exit logging, if any data remains in the buffer, write to EEPROM
			if (eepromBufferCounter >= 1) {
				soft_i2c_eeprom_page_write(EEPROM_ADDR, eepromPageCounter, eepromPageByteCounter, eepromBuffer, eepromBufferCounter);
			}
			
			// Turn off 1Hz clock output
			soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_CONTROL, NOREPEATSTART);
			soft_i2c_master_write(RTC_CONTROL_POR_BITS);
			soft_i2c_master_stop();
			
			// Enable pin change on 3.3V LDO sense and disable RTC 1Hz sense
			sbi(PCMSK0, PCINT2);
			cbi(PCMSK0, PCINT3);
		}
		else { // Not logging, sleep
			system_sleep();
		}
	}
	
	return 0;
}

EMPTY_INTERRUPT(WDT_vect);

ISR(PCINT0_vect) {
	// Check if button is being held down for 2 seconds, if it is, switch the state of logging variable
	uint8_t pinHeldCounter = 0;
	while (PINA & (1<<buttonPin)) {
		_delay_ms(20); // Multiply by 8 for real delay ms
		pinHeldCounter++;
		
		if (pinHeldCounter >= 12) { // 2 seconds
			if (logging == false) {
				logging = true;
			}
			else {
				logging = false;
			}
			PORTA |= (1<<ledPin); // Turn on LED
			_delay_ms(50); // Multiply by 8 for real delay ms
			PORTA &= ~(1<<ledPin); // Turn off
			break;
		}
	}
	
	// Wait for button release (so we don't generate another interrupt when released)
	while (PINA & (1<<buttonPin)) {
		_delay_ms(20); // Multiply by 8 for real delay ms
	}
}
