/*
 Mini Temp Logger
 Version: Prototype R4
 Author: Alex from insideGadgets (www.insidegadgets.com)
 Created: 22/02/2015
 Last Modified: 11/09/2016
 
 */

// Common defines
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

#ifndef NULL
#define NULL ((void *)0)
#endif

#define LOW 0
#define HIGH 1
#define false 0
#define true 1

// Watchdog timeouts to values
#define T16MS 0
#define T32MS 1
#define T64MS 2
#define T128MS 3
#define T250MS 4
#define T500MS 5
#define T1S 6
#define T2S 7
#define T4S 8
#define T8S 9

// I/O
#define ledPin PA7
#define buttonPin PA6
#define usbSensePin PA2
#define TWI_SDA_PIN PA4
#define TWI_SCL_PIN PA5

// General variables and constants
#define delayTimeLocation 1 // 1 - 4, Where in the ATtiny EEPROM we store the 4 bytes that give us our delay time
#define rtcTimeLocation 8 // 5 - 11, RTC time stored we start logging
#define rtcTrimLocation 16 // 12, Trim value used if the ATtiny is restarted

// EEPROM settings
#define EEPROM_ADDR 0x50 // I2C EEPROM address
#define EEPROM_BUFFER_LENGTH 32
uint8_t eepromBufferCounter = 0;
uint8_t eepromBuffer[EEPROM_BUFFER_LENGTH];
uint16_t eepromPageCounter = 0;
uint8_t eepromPageByteCounter = 0;

static volatile uint8_t logging = 0;
static uint32_t dataCount = 0;
static uint32_t endCount = 131071;

#define blankEeprombyte 255
#define blankEeprom2byte 57081 // If high and low byte in EEPROM combined equals this, it means the 2 bytes have no valid data
#define NOREPEATSTART 0
#define REPEATSTART 1

// I2C settings
#define I2C_DELAY_CYCLES_BAT 1 // 2 us Running at 1.5MHz
#define I2C_DELAY_CYCLES_USB 20 // 5 us Running at 12MHz
uint8_t I2C_DELAY_CYCLES = I2C_DELAY_CYCLES_BAT;

#define I2C_READ 1
#define I2C_WRITE 0

// TMP102
#define TEMP_ADDR 0x48 // I2C TMP102 address
#define TMP102_TEMP_REG 0x00
#define TMP102_CONFIG_REG 0x01
#define TMP102_CONFIG_SD_MODE 0x01
#define TMP102_CONFIG_EM 0x10
#define TMP102_CONFIG_ONESHOT 0x80

// RTC
#define RTC_ADDRESS 0x6F
#define RTC_RTCSEC 0x00
#define RTC_RTCSEC_ST_BIT 0x80
#define RTC_RTCMIN 0x01
#define RTC_RTCHOUR 0x02
#define RTC_RTCHOUR_TIMEFORMAT_BIT 0x40
#define RTC_RTCDATE 0x04
#define RTC_RTCMTH 0x05
#define RTC_RTCYEAR 0x06
#define RTC_CONTROL 0x07
#define RTC_CONTROL_SQWEN_BIT 0x40
#define RTC_CONTROL_POR_BITS 0x80
#define RTC_TRIM 0x08
uint8_t rtcTrim = 0;
uint8_t rtcSec = 0;
uint8_t rtcMin = 0;
uint8_t rtcAmPm = 0;
uint8_t rtcHr = 0;
uint8_t rtcDate = 0;
uint8_t rtcMon = 0;
uint8_t rtcYear = 0;

// USB transfer variables
#define USB_TRANSFER 1
#define USB_DATAFROMPC 2
#define USB_DELAYTIMESEND 3
#define USB_RTCSTARTLOGTIMESEND 4
#define USB_RTCCURRENTTIMESEND 5
#define USB_RTCTRIMSEND 6
#define USB_TEMPSEND 7
uchar replyBuf[254];
static uchar *nextDigit;
static uchar dataReceived = 0, dataLength = 0; // for USB_DATAFROMPC
uint8_t inUSB = false;

// Sub function selection when sending data from the PC
#define PROGRAM_DELAY_TIME 1
#define PROGRAM_RTC_TIME 2
#define PROGRAM_RTC_TRIM 3

// Put the ATtiny in power down sleep mode
void system_sleep(void) {
	set_sleep_mode(SLEEP_MODE_PWR_DOWN); // Set sleep mode
	sleep_mode(); // System sleeps here
}

// Watchdog timeout values
// 0=16ms, 1=32ms, 2=64ms, 3=128ms, 4=250ms, 5=500ms, 6=1sec, 7=2sec, 8=4sec, 9=8sec
void setup_watchdog(uint8_t wdDelay) {
	if (wdDelay > 9) { // Set max as 8 sec
		wdDelay = 9;
	}
	if (wdDelay > 7) { // Set WDP3 bit
		wdDelay = wdDelay & 7;
		wdDelay |= (1<<5);
	}
	
	MCUSR &= ~(1<<WDRF);
	// Start timed sequence
	WDTCSR |= (1<<WDCE) | (1<<WDE);
	// Set new watchdog timeout value
	WDTCSR = wdDelay | (1<<WDIE);
}

// Turn off the Watchdog
void turn_off_watchdog(void) {
	cli(); // Turn interrupts off
	MCUSR &= ~(1<<WDRF);
	WDTCSR |= (1<<WDCE) | (1<<WDE);
	WDTCSR = 0;
	sei(); // Turn interrupts back on
}

// Watchdog sleep function - combines all the above functions into one
void watchdog_sleep(uint8_t timeout) { 
	setup_watchdog(timeout);
	system_sleep();
	turn_off_watchdog();
}

// Wait for RTC 1Hz pulse high and low (500ms length each)
void rtc_sleep_1_sec(void) {
	system_sleep();
	_delay_us(10); // x8 clock = 80us
	
	system_sleep();
	_delay_us(10); // x8 clock = 80us
}

// Read a byte from I2C and send Ack if more reads follow else Nak to terminate read
uint8_t soft_i2c_master_read(uint8_t last) {
	uint8_t b = 0;
	for (uint8_t i = 0; i < 8; i++) {
		// Don't change this loop unless you verify the change with a scope
		b <<= 1;
		_delay_loop_1(I2C_DELAY_CYCLES);
		DDRA &= ~(1<<TWI_SCL_PIN);
		_delay_loop_1(I2C_DELAY_CYCLES);
		if (bit_is_set(PINA, TWI_SDA_PIN)) b |= 1;
		_delay_loop_1(I2C_DELAY_CYCLES);
		DDRA |= (1<<TWI_SCL_PIN);
	}
	_delay_loop_1(I2C_DELAY_CYCLES);
	
	// Send Ack or Nak
	if (last) {
		DDRA &= ~(1<<TWI_SDA_PIN);
	}
	else {
		DDRA |= (1<<TWI_SDA_PIN);
	}

	DDRA &= ~(1<<TWI_SCL_PIN);
	_delay_loop_1(I2C_DELAY_CYCLES);
	DDRA |= (1<<TWI_SCL_PIN);
	DDRA &= ~(1<<TWI_SDA_PIN);
	
	return b;
}

// Write a byte to I2C
uint8_t soft_i2c_master_write(uint8_t data) {
	uint8_t rtn = 0;
	
	// If powered from battery, make things a bit quicker
	if (I2C_DELAY_CYCLES == I2C_DELAY_CYCLES_BAT) {
		// Write byte
		for (uint8_t m = 0x80; m != 0; m >>= 1) {
			// Don't change this loop unless you verify the change with a scope
			if (m & data) { 
				DDRA &= ~(1<<TWI_SDA_PIN);
			}
			else { 
				DDRA |= (1<<TWI_SDA_PIN);
			}
			
			DDRA &= ~(1<<TWI_SCL_PIN);
			asm ("nop");
			DDRA |= (1<<TWI_SCL_PIN);
		}
		
		DDRA &= ~(1<<TWI_SDA_PIN);
		DDRA &= ~(1<<TWI_SCL_PIN);
		asm ("nop");
		
		// get Ack or Nak
		rtn = bit_is_set(PINA, TWI_SDA_PIN);
		
		DDRA |= (1<<TWI_SCL_PIN);
		DDRA &= ~(1<<TWI_SDA_PIN);
	}
	else {
		// Write byte
		for (uint8_t m = 0x80; m != 0; m >>= 1) {
			// Don't change this loop unless you verify the change with a scope
			if (m & data) { 
				DDRA &= ~(1<<TWI_SDA_PIN);
			}
			else { 
				DDRA |= (1<<TWI_SDA_PIN);
			}
			
			_delay_loop_1(I2C_DELAY_CYCLES);
			DDRA &= ~(1<<TWI_SCL_PIN);
			_delay_loop_1(I2C_DELAY_CYCLES);
			DDRA |= (1<<TWI_SCL_PIN);
			_delay_loop_1(I2C_DELAY_CYCLES);
		}
		_delay_loop_1(I2C_DELAY_CYCLES);
		
		DDRA &= ~(1<<TWI_SDA_PIN);
		DDRA &= ~(1<<TWI_SCL_PIN);
		_delay_loop_1(I2C_DELAY_CYCLES);
		
		// get Ack or Nak
		rtn = bit_is_set(PINA, TWI_SDA_PIN);
		
		DDRA |= (1<<TWI_SCL_PIN);
		DDRA &= ~(1<<TWI_SDA_PIN);
		_delay_loop_1(I2C_DELAY_CYCLES);
	}
	
	return rtn == 0;
}

// Issue a start condition
uint8_t soft_i2c_master_start(uint8_t addressRW) {
	DDRA |= (1<<TWI_SDA_PIN);
	_delay_loop_1(I2C_DELAY_CYCLES);
	DDRA |= (1<<TWI_SCL_PIN);
	_delay_loop_1(I2C_DELAY_CYCLES);
	
	return soft_i2c_master_write(addressRW);
}

// Issue a restart condition
uint8_t soft_i2c_master_restart(uint8_t addressRW) {
	DDRA &= ~(1<<TWI_SDA_PIN);
	DDRA &= ~(1<<TWI_SCL_PIN);
	_delay_loop_1(I2C_DELAY_CYCLES);
	
	return soft_i2c_master_start(addressRW);
}

// Issue a stop condition
void soft_i2c_master_stop(void) {
	DDRA |= (1<<TWI_SDA_PIN);
	_delay_loop_1(I2C_DELAY_CYCLES);
	
	DDRA &= ~(1<<TWI_SCL_PIN);
	_delay_loop_1(I2C_DELAY_CYCLES);
	
	DDRA &= ~(1<<TWI_SDA_PIN);
	_delay_loop_1(I2C_DELAY_CYCLES);
}

// Issue the device start and the 8bit address to write
uint8_t soft_i2c_master_start_with_8bit_address(uint8_t deviceAddr, uint16_t addressRW, uint8_t repeatedStart) {
	if (!soft_i2c_master_start((deviceAddr<<1) | I2C_WRITE)) return false;
	if (!soft_i2c_master_write(addressRW)) return false;
	if (repeatedStart) { if (!soft_i2c_master_restart((deviceAddr<<1) | I2C_READ)) return false; }
	
	return true;
}

// Issue the device start and the address to write
uint8_t soft_i2c_master_start_with_address(uint8_t deviceAddr, uint16_t addressRW, uint8_t repeatedStart) {
	if (!soft_i2c_master_start((deviceAddr<<1) | I2C_WRITE)) return false;
	if (!soft_i2c_master_write((addressRW >> 8))) return false; // MSB
	if (!soft_i2c_master_write((addressRW & 0xFF))) return false; // LSB
	if (repeatedStart) { if (!soft_i2c_master_restart((deviceAddr<<1) | I2C_READ)) return false; }
	
	return true;
}

// Read 252 bytes from the EEPROM device and put in the reply buffer
uint8_t soft_i2c_eeprom_read_bytes(uint8_t deviceAddr, uint32_t readAddress) {
	// Transfer bit 16 in Writeaddress to deviceAddr
	if (readAddress & 0x10000) {
		deviceAddr |= (1<<0);
	}
	uint16_t processedAddress = readAddress & 0xFFFF;
	
	// Issue start condition, send device address, issue repeated start
	soft_i2c_master_start_with_address(deviceAddr, processedAddress, REPEATSTART);
	
	// Read the bytes
	for (uint8_t x = 0; x < 252; x++) {
		replyBuf[x] = soft_i2c_master_read(0);
	}
	soft_i2c_master_read(1);
	
	// Issue a stop condition
	soft_i2c_master_stop();
	
	return 1;
}

// Write 1 byte to the EEPROM
uint8_t soft_i2c_eeprom_write_byte(uint8_t deviceAddr, uint32_t writeAddress, uint8_t writeByte) {
	// Transfer bit 16 in Writeaddress to deviceAddr
	if (writeAddress & 0x10000) {
		deviceAddr |= (1<<0);
	}
	uint16_t processedAddress = writeAddress & 0xFFFF;
	
	// Issue start condition and send device address
	soft_i2c_master_start_with_address(deviceAddr, processedAddress, NOREPEATSTART);
	
	// Write the byte
	if (!soft_i2c_master_write(writeByte)) return false;
	
	// Issue a stop condition
	soft_i2c_master_stop();
	
	return true;
}

// Page write x bytes to the EEPROM
uint8_t soft_i2c_eeprom_page_write(uint8_t deviceAddr, uint16_t pageAddress, uint8_t byteAddress, uint8_t* writeByte, uint8_t writeLength) {
	// If bit 9 is set in page address, set slave Address byte high
	if (pageAddress & 0x100) {
		deviceAddr |= (1<<0);
		pageAddress = pageAddress & 0xFF; // Clear high 8 bits
	}
	
	// Issue start condition and send device address
	soft_i2c_master_start_with_address(deviceAddr, (pageAddress << 8) | byteAddress, NOREPEATSTART);
	
	// Write all the bytes
	for (uint8_t x = 0; x < writeLength; x++) {
		soft_i2c_master_write(writeByte[x]);
	}
	
	// Issue a stop condition
	soft_i2c_master_stop();
	
	return true;
}

// Page write blank bytes (0xFF) to 1 page (256 bytes) on the EEPROM
void soft_i2c_eeprom_page_blank(uint8_t deviceAddr, uint16_t pageAddress) {
	// If bit 9 is set in page, set slave Address byte high
	if (pageAddress & 0x100) {
		deviceAddr |= (1<<0);
		pageAddress = pageAddress & 0xFF; // Clear 9th bit
	}
	
	// Issue start condition and send device address
	soft_i2c_master_start_with_address(deviceAddr, (pageAddress << 8), NOREPEATSTART);
	
	// Write all 0xFF
	for (uint16_t x = 0; x < 256; x++) {
		soft_i2c_master_write(0xFF);
	}
	
	// Issue a stop condition
	soft_i2c_master_stop();
}

// Set TMP102 configuration to 13bit and shutdown mode (SD) on
uint8_t tmp102_set_config(uint8_t deviceAddr) {
	if (!soft_i2c_master_start((deviceAddr<<1) | I2C_WRITE)) return false;
	if (!soft_i2c_master_write(TMP102_CONFIG_REG)) return false;
	if (!soft_i2c_master_write(TMP102_CONFIG_SD_MODE)) return false;
	if (!soft_i2c_master_write(TMP102_CONFIG_EM)) return false;
	soft_i2c_master_stop();
	
	return true;
}

// Read the TMP102 temperature
int tmp102_read_temp(uint8_t deviceAddr) {
	// Turn on one shot (plus re-set to 13bit and shutdown mode (SD))
	if (!soft_i2c_master_start((deviceAddr<<1) | I2C_WRITE)) return false;
	if (!soft_i2c_master_write(TMP102_CONFIG_REG)) return false;
	if (!soft_i2c_master_write(TMP102_CONFIG_SD_MODE | TMP102_CONFIG_ONESHOT)) return false;
	if (!soft_i2c_master_write(TMP102_CONFIG_EM)) return false;
	soft_i2c_master_stop();
	
	// Wait 32ms for the temperature conversion to complete
	if (inUSB == false) {
		watchdog_sleep(T32MS);
	}
	else { // Whilst USB is plugged in, can't use the sleep routine
		_delay_ms(40);
	}
	
	// Read temperature and combine bytes
	if (!soft_i2c_master_start((deviceAddr<<1) | I2C_WRITE)) return false;
	if (!soft_i2c_master_write(TMP102_TEMP_REG)) return false;
	if (!soft_i2c_master_restart((deviceAddr<<1) | I2C_READ)) return false;
	
	uint8_t low, high;
	high = soft_i2c_master_read(0);
	low = soft_i2c_master_read(1);
	soft_i2c_master_stop();
	
	int combinedResult = ((high << 8) | low) >> 3;
	
	return combinedResult;
}

// Save the current RTC time to the EEPROM
void save_rtc_time_to_eeprom(void) {
	soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_RTCSEC, REPEATSTART);
	uint8_t rtcTimeSeconds = soft_i2c_master_read(1);
	soft_i2c_master_stop();
	
	soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_RTCMIN, REPEATSTART);
	uint8_t rtcTimeMinutes = soft_i2c_master_read(1);
	soft_i2c_master_stop();
	
	soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_RTCHOUR, REPEATSTART);
	uint8_t rtcTimeHour = soft_i2c_master_read(1);
	soft_i2c_master_stop();
	
	soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_RTCDATE, REPEATSTART);
	uint8_t rtcTimeDate = soft_i2c_master_read(1);
	soft_i2c_master_stop();
	
	soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_RTCMTH, REPEATSTART);
	uint8_t rtcTimeMonth = soft_i2c_master_read(1);
	soft_i2c_master_stop();
	
	soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_RTCYEAR, REPEATSTART);
	uint8_t rtcTimeYear = soft_i2c_master_read(1);
	soft_i2c_master_stop();
	
	// Convert MCP7940M format time to normal time
	rtcAmPm = (rtcTimeHour & 0x20) >> 5;
	rtcSec = (((rtcTimeSeconds & 0x7F) >> 4) * 10) + (rtcTimeSeconds & 0x0F);
	rtcMin = (((rtcTimeMinutes & 0x7F) >> 4) * 10) + (rtcTimeMinutes & 0x0F);
	rtcHr = (((rtcTimeHour & 0x10) >> 4) * 10) + (rtcTimeHour & 0x0F);
	rtcDate = (((rtcTimeDate & 0x30) >> 4) * 10) + (rtcTimeDate & 0x0F);
	rtcMon = (((rtcTimeMonth & 0x10) >> 4) * 10) + (rtcTimeMonth & 0x0F);
	rtcYear = (((rtcTimeYear & 0xF0) >> 4) * 10) + (rtcTimeYear & 0x0F);
	
	eeprom_write_byte((uint8_t*) rtcTimeLocation, rtcAmPm);
	eeprom_write_byte((uint8_t*) rtcTimeLocation+1, rtcSec); 
	eeprom_write_byte((uint8_t*) rtcTimeLocation+2, rtcMin); 
	eeprom_write_byte((uint8_t*) rtcTimeLocation+3, rtcHr); 
	eeprom_write_byte((uint8_t*) rtcTimeLocation+4, rtcDate);
	eeprom_write_byte((uint8_t*) rtcTimeLocation+5, rtcMon);
	eeprom_write_byte((uint8_t*) rtcTimeLocation+6, rtcYear);
}

// Read the current RTC time and store to our variables
void read_rtc_time(void) {
	soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_RTCSEC, REPEATSTART);
	uint8_t rtcTimeSeconds = soft_i2c_master_read(1);
	soft_i2c_master_stop();
	
	soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_RTCMIN, REPEATSTART);
	uint8_t rtcTimeMinutes = soft_i2c_master_read(1);
	soft_i2c_master_stop();
	
	soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_RTCHOUR, REPEATSTART);
	uint8_t rtcTimeHour = soft_i2c_master_read(1);
	soft_i2c_master_stop();
	
	soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_RTCDATE, REPEATSTART);
	uint8_t rtcTimeDate = soft_i2c_master_read(1);
	soft_i2c_master_stop();
	
	soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_RTCMTH, REPEATSTART);
	uint8_t rtcTimeMonth = soft_i2c_master_read(1);
	soft_i2c_master_stop();
	
	soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_RTCYEAR, REPEATSTART);
	uint8_t rtcTimeYear = soft_i2c_master_read(1);
	soft_i2c_master_stop();
	
	// Convert MCP7940M format time to normal time
	rtcAmPm = (rtcTimeHour & 0x20) >> 5;
	rtcSec = (((rtcTimeSeconds & 0x7F) >> 4) * 10) + (rtcTimeSeconds & 0x0F);
	rtcMin = (((rtcTimeMinutes & 0x7F) >> 4) * 10) + (rtcTimeMinutes & 0x0F);
	rtcHr = (((rtcTimeHour & 0x10) >> 4) * 10) + (rtcTimeHour & 0x0F);
	rtcDate = (((rtcTimeDate & 0x30) >> 4) * 10) + (rtcTimeDate & 0x0F);
	rtcMon = (((rtcTimeMonth & 0x10) >> 4) * 10) + (rtcTimeMonth & 0x0F);
	rtcYear = (((rtcTimeYear & 0xF0) >> 4) * 10) + (rtcTimeYear & 0x0F);
}

// Setup the ATtiny (has 8 clock prescaler at startup)
void setup(void) {
	// Set Outputs
	DDRA |= (1<<ledPin);
	PORTA &= ~(1<<ledPin);
	
	// Turn on pull-up resistors on other ports to save power
	//PORTA |= (1<<PB0);
	
	// Turn on interrupts
	sei();
	
	// Wait a little bit for things to settle
	watchdog_sleep(T250MS);
	
	// Initialse I2C
	DDRA &= ~(1<<TWI_SCL_PIN);
	PORTA &= ~(1<<TWI_SCL_PIN);
	DDRA &= ~(1<<TWI_SDA_PIN);
	PORTA &= ~(1<<TWI_SDA_PIN);
	
	// Setup TMP102
	tmp102_set_config(TEMP_ADDR);
	
	// Set RTC trim if it's set
	uint8_t readRtcTrim = eeprom_read_byte((uint8_t*) rtcTrimLocation);
	if (readRtcTrim != blankEeprombyte) {
		rtcTrim = readRtcTrim;
		soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_TRIM, NOREPEATSTART);
		soft_i2c_master_write(rtcTrim);
		soft_i2c_master_stop();
	}
	
	// Setup pin change interrupt on 3.3V LDO Sense & button
	sbi(GIMSK, PCIE0);
	sbi(PCMSK0, PCINT2);
	sbi(PCMSK0, PCINT6);
	
	// Turn on LED for 2 seconds to show all is well
	PORTA |= (1<<ledPin);
	watchdog_sleep(T2S);
	PORTA &= ~(1<<ledPin);
}

// Start USB communications
void startUSB(void) {
	usbDeviceDisconnect();
	_delay_ms(300); // 300 ms disconnect
	usbDeviceConnect();
	usbInit();
}

// This gets called when custom control message is received
USB_PUBLIC uchar usbFunctionSetup(uchar data[8]) {
	usbRequest_t *rq = (void *)data; // cast data to correct type
	
	if (rq->bRequest == USB_TRANSFER) { // custom command is in the bRequest field
		if (dataCount <= endCount) {
			soft_i2c_eeprom_read_bytes(EEPROM_ADDR, dataCount);
			dataCount += 252;
		}
		
		usbMsgPtr = replyBuf;
		
		return sizeof(replyBuf);
	}
	else if (rq->bRequest == USB_DATAFROMPC) {
		dataLength = (uchar)rq->wLength.word;
		dataReceived = 0;
		
		if (dataLength > sizeof(replyBuf)) { // Limit to buffer size
			dataLength = sizeof(replyBuf);
		}
		
		return USB_NO_MSG; // usbFunctionWrite will be called now
	}
	else if (rq->bRequest == USB_DELAYTIMESEND) { // Transfer delay time and reset data count
		dataCount = 0;
		
		nextDigit = &replyBuf[-1];
		*++nextDigit = eeprom_read_byte((uint8_t*) delayTimeLocation); // Seconds 
		*++nextDigit = eeprom_read_byte((uint8_t*) delayTimeLocation+1); // Minutes
		*++nextDigit = eeprom_read_byte((uint8_t*) delayTimeLocation+2); // Hours
		*++nextDigit = eeprom_read_byte((uint8_t*) delayTimeLocation+3); // Days
		
		*++nextDigit = '\0';
		usbMsgPtr = replyBuf;
		return sizeof(replyBuf);
	}
	else if (rq->bRequest == USB_RTCSTARTLOGTIMESEND) { // Transfer the RTC time when the logging started
		nextDigit = &replyBuf[-1];
		*++nextDigit = eeprom_read_byte((uint8_t*) rtcTimeLocation); // AM/PM
		*++nextDigit = eeprom_read_byte((uint8_t*) rtcTimeLocation+1); // Seconds
		*++nextDigit = eeprom_read_byte((uint8_t*) rtcTimeLocation+2); // Minutes
		*++nextDigit = eeprom_read_byte((uint8_t*) rtcTimeLocation+3); // Hour
		*++nextDigit = eeprom_read_byte((uint8_t*) rtcTimeLocation+4); // Day
		*++nextDigit = eeprom_read_byte((uint8_t*) rtcTimeLocation+5); // Month
		*++nextDigit = eeprom_read_byte((uint8_t*) rtcTimeLocation+6); // Year
		
		*++nextDigit = '\0';
		usbMsgPtr = replyBuf;
		return sizeof(replyBuf);
	}
	else if (rq->bRequest == USB_RTCCURRENTTIMESEND) { // Transfer the current RTC time
		nextDigit = &replyBuf[-1];
		
		read_rtc_time();
		*++nextDigit = rtcAmPm; // AM/PM
		*++nextDigit = rtcSec; // Seconds
		*++nextDigit = rtcMin; // Minutes
		*++nextDigit = rtcHr; // Hour
		*++nextDigit = rtcDate; // Day
		*++nextDigit = rtcMon; // Month
		*++nextDigit = rtcYear; // Year
		
		*++nextDigit = '\0';
		usbMsgPtr = replyBuf;
		return sizeof(replyBuf);
	}
	else if (rq->bRequest == USB_RTCTRIMSEND) { // Transfer RTC TRIM back to PC
		soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_TRIM, REPEATSTART);
		rtcTrim = soft_i2c_master_read(1);
		soft_i2c_master_stop();
		
		nextDigit = &replyBuf[-1];
		*++nextDigit = rtcTrim;
		*++nextDigit = '\0';
		usbMsgPtr = replyBuf;
		return sizeof(replyBuf);
	}
	else if (rq->bRequest == USB_TEMPSEND) { // Read current temperature
		int tempResult = tmp102_read_temp(TEMP_ADDR);
		
		nextDigit = &replyBuf[-1];
		*++nextDigit = (tempResult >> 8);
		*++nextDigit = (tempResult & 0xFF);
		*++nextDigit = '\0';
		usbMsgPtr = replyBuf;
		
		return sizeof(replyBuf);
	}
	
	return 0;
}

// This gets called when data is sent from PC to the device
USB_PUBLIC uchar usbFunctionWrite(uchar *data, uchar len) {	
	for (uchar i = 0; dataReceived < dataLength && i < len; i++, dataReceived++) {
		replyBuf[dataReceived] = data[i];
	}
	
	// Re-program the delay time - Read delay time and write to ATtiny EEPROM
	if (replyBuf[0] == PROGRAM_DELAY_TIME) {
		uint8_t delayTimeProgramSeconds = replyBuf[1];
		uint8_t delayTimeProgramMinutes = replyBuf[2];
		uint8_t delayTimeProgramHours = replyBuf[3];
		uint8_t delayTimeProgramDays = replyBuf[4];
		
		eeprom_write_byte((uint8_t*) delayTimeLocation, delayTimeProgramSeconds); // Seconds
		eeprom_write_byte((uint8_t*) delayTimeLocation+1, delayTimeProgramMinutes); // Minutes
		eeprom_write_byte((uint8_t*) delayTimeLocation+2, delayTimeProgramHours); // Hours
		eeprom_write_byte((uint8_t*) delayTimeLocation+3, delayTimeProgramDays); // Days
	}
	else if (replyBuf[0] == PROGRAM_RTC_TIME) { // Write the RTC time to the EEPROM, change the rtc format to suit MCP7940M and set start RTC bit, stop the RTC then re-program the RTC time
		rtcAmPm = replyBuf[1];
		rtcSec = replyBuf[2];
		rtcMin = replyBuf[3];
		rtcHr = replyBuf[4];
		rtcDate = replyBuf[5];
		rtcMon = replyBuf[6];
		rtcYear = replyBuf[7];
		
		// Change RTC time to MCP7940M format
		if (rtcSec > 10) {
			rtcSec = ((rtcSec / 10) << 4) | (rtcSec % 10);
		}
		rtcSec |= RTC_RTCSEC_ST_BIT; // Set RTC start bit
		if (rtcMin > 10) {
			rtcMin = ((rtcMin / 10) << 4) | (rtcMin % 10);
		}
		if (rtcHr > 10) {
			rtcHr = (1<<4) | (rtcHr % 10);
		}
		rtcHr |= (1<<6) | (rtcAmPm<<5); // Set 12 hour time and AM/PM bit if needed
		if (rtcDate > 10) {
			rtcDate = ((rtcDate / 10) << 4) | (rtcDate % 10);
		}
		if (rtcMon > 10) {
			rtcMon = (1<<4) | (rtcMon % 10);
		}
		if (rtcYear > 10) {
			rtcYear = ((rtcYear / 10) << 4) | (rtcYear % 10);
		}
		
		// Stop RTC
		soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_RTCSEC, NOREPEATSTART);
		soft_i2c_master_write(0);
		soft_i2c_master_stop();
		
		// Re-program RTC time (seconds last as it turns on the RTC)
		soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_RTCYEAR, NOREPEATSTART);
		soft_i2c_master_write(rtcYear);
		soft_i2c_master_stop();
		soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_RTCMTH, NOREPEATSTART);
		soft_i2c_master_write(rtcMon);
		soft_i2c_master_stop();
		soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_RTCDATE, NOREPEATSTART);
		soft_i2c_master_write(rtcDate);
		soft_i2c_master_stop();
		soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_RTCHOUR, NOREPEATSTART);
		soft_i2c_master_write(rtcHr);
		soft_i2c_master_stop();
		soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_RTCMIN, NOREPEATSTART);
		soft_i2c_master_write(rtcMin);
		soft_i2c_master_stop();
		soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_RTCSEC, NOREPEATSTART);
		soft_i2c_master_write(rtcSec);
		soft_i2c_master_stop();
	}
	else if (replyBuf[0] == PROGRAM_RTC_TRIM) { // Write the RTC TRIM to EEPROM and RTC
		rtcTrim = replyBuf[1];
		
		eeprom_write_byte((uint8_t*) rtcTrimLocation, rtcTrim);
		
		soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_TRIM, NOREPEATSTART);
		soft_i2c_master_write(rtcTrim);
		soft_i2c_master_stop();
	}
	
	PORTA &= ~(1<<ledPin);
	_delay_ms(10);
	PORTA |= (1<<ledPin);
	
	return (dataReceived == dataLength); // 1 if we received it all, 0 if not
}
